import type { ConfigEntry, TraversalEntry, TraversalKey, TraversalNode, TraversalValue } from "../nodes/node.js";
import type { TraversalProp } from "../nodes/rules/props.js";
import type { Scope } from "../scopes/scope.js";
import type { Type, TypeConfig } from "../scopes/type.js";
import type { SizedData } from "../utils/data.js";
import type { extend, stringKeyOf } from "../utils/generics.js";
import { Path } from "../utils/paths.js";
import type { ProblemCode, ProblemWriters } from "./problems.js";
import { Problem } from "./problems.js";
export declare class TraversalState<data = unknown> {
    #private;
    data: data;
    type: Type;
    path: Path;
    problems: import("../utils/generics.js").arraySubclassToReadonly<{
        [n: number]: Problem<any>;
        byPath: Record<string, Problem<any>>;
        count: number;
        "__#2@#state": TraversalState<unknown>;
        mustBe(description: string, opts?: import("./problems.js").AddProblemOptions<unknown> | undefined): Problem<any>;
        add<code extends ProblemCode>(code: code, source: import("./problems.js").ProblemSource<code>, opts?: import("./problems.js").AddProblemOptions<import("./problems.js").ProblemData<code>> | undefined): Problem<any>;
        addProblem(problem: Problem<any>): void;
        readonly summary: string;
        toString(): string;
        throw(): never;
        length: number;
        toLocaleString(): string;
        pop(): Problem<any> | undefined;
        push(...items: Problem<any>[]): number;
        concat(...items: ConcatArray<Problem<any>>[]): Problem<any>[];
        concat(...items: (Problem<any> | ConcatArray<Problem<any>>)[]): Problem<any>[];
        join(separator?: string | undefined): string;
        reverse(): Problem<any>[];
        shift(): Problem<any> | undefined;
        slice(start?: number | undefined, end?: number | undefined): Problem<any>[];
        sort(compareFn?: ((a: Problem<any>, b: Problem<any>) => number) | undefined): any;
        splice(start: number, deleteCount?: number | undefined): Problem<any>[];
        splice(start: number, deleteCount: number, ...items: Problem<any>[]): Problem<any>[];
        unshift(...items: Problem<any>[]): number;
        indexOf(searchElement: Problem<any>, fromIndex?: number | undefined): number;
        lastIndexOf(searchElement: Problem<any>, fromIndex?: number | undefined): number;
        every<S extends Problem<any>>(predicate: (value: Problem<any>, index: number, array: Problem<any>[]) => value is S, thisArg?: any): this is S[];
        every(predicate: (value: Problem<any>, index: number, array: Problem<any>[]) => unknown, thisArg?: any): boolean;
        some(predicate: (value: Problem<any>, index: number, array: Problem<any>[]) => unknown, thisArg?: any): boolean;
        forEach(callbackfn: (value: Problem<any>, index: number, array: Problem<any>[]) => void, thisArg?: any): void;
        map<U>(callbackfn: (value: Problem<any>, index: number, array: Problem<any>[]) => U, thisArg?: any): U[];
        filter<S_1 extends Problem<any>>(predicate: (value: Problem<any>, index: number, array: Problem<any>[]) => value is S_1, thisArg?: any): S_1[];
        filter(predicate: (value: Problem<any>, index: number, array: Problem<any>[]) => unknown, thisArg?: any): Problem<any>[];
        reduce(callbackfn: (previousValue: Problem<any>, currentValue: Problem<any>, currentIndex: number, array: Problem<any>[]) => Problem<any>): Problem<any>;
        reduce(callbackfn: (previousValue: Problem<any>, currentValue: Problem<any>, currentIndex: number, array: Problem<any>[]) => Problem<any>, initialValue: Problem<any>): Problem<any>;
        reduce<U_1>(callbackfn: (previousValue: U_1, currentValue: Problem<any>, currentIndex: number, array: Problem<any>[]) => U_1, initialValue: U_1): U_1;
        reduceRight(callbackfn: (previousValue: Problem<any>, currentValue: Problem<any>, currentIndex: number, array: Problem<any>[]) => Problem<any>): Problem<any>;
        reduceRight(callbackfn: (previousValue: Problem<any>, currentValue: Problem<any>, currentIndex: number, array: Problem<any>[]) => Problem<any>, initialValue: Problem<any>): Problem<any>;
        reduceRight<U_2>(callbackfn: (previousValue: U_2, currentValue: Problem<any>, currentIndex: number, array: Problem<any>[]) => U_2, initialValue: U_2): U_2;
        find<S_2 extends Problem<any>>(predicate: (value: Problem<any>, index: number, obj: Problem<any>[]) => value is S_2, thisArg?: any): S_2 | undefined;
        find(predicate: (value: Problem<any>, index: number, obj: Problem<any>[]) => unknown, thisArg?: any): Problem<any> | undefined;
        findIndex(predicate: (value: Problem<any>, index: number, obj: Problem<any>[]) => unknown, thisArg?: any): number;
        fill(value: Problem<any>, start?: number | undefined, end?: number | undefined): any;
        copyWithin(target: number, start: number, end?: number | undefined): any;
        entries(): IterableIterator<[number, Problem<any>]>;
        keys(): IterableIterator<number>;
        values(): IterableIterator<Problem<any>>;
        includes(searchElement: Problem<any>, fromIndex?: number | undefined): boolean;
        flatMap<U_3, This = undefined>(callback: (this: This, value: Problem<any>, index: number, array: Problem<any>[]) => U_3 | readonly U_3[], thisArg?: This | undefined): U_3[];
        flat<A, D extends number = 1>(this: A, depth?: D | undefined): FlatArray<A, D>[];
        at(index: number): Problem<any> | undefined;
        findLast<S_3 extends Problem<any>>(predicate: (value: Problem<any>, index: number, array: Problem<any>[]) => value is S_3, thisArg?: any): S_3 | undefined;
        findLast(predicate: (value: Problem<any>, index: number, array: Problem<any>[]) => unknown, thisArg?: any): Problem<any> | undefined;
        findLastIndex(predicate: (value: Problem<any>, index: number, array: Problem<any>[]) => unknown, thisArg?: any): number;
        [Symbol.iterator](): IterableIterator<Problem<any>>;
        [Symbol.unscopables](): {
            copyWithin: boolean;
            entries: boolean;
            fill: boolean;
            find: boolean;
            findIndex: boolean;
            keys: boolean;
            values: boolean;
        };
    }>;
    failFast: boolean;
    traversalConfig: TraversalConfig;
    readonly rootScope: Scope;
    constructor(data: data, type: Type);
    getProblemConfig<code extends ProblemCode>(code: code): ProblemWriters<code>;
    getConfigKey<k extends keyof TypeConfig>(k: k): {
        keys?: import("../scopes/type.js").KeyCheckKind;
        mustBe?: import("./problems.js").MustBeWriter<ProblemCode>;
        writeReason?: import("./problems.js").ReasonWriter<ProblemCode>;
        addContext?: import("./problems.js").ContextWriter;
    }[k] | undefined;
    traverseConfig(configEntries: ConfigEntry[], node: TraversalNode): boolean;
    traverseKey(key: stringKeyOf<this["data"]>, node: TraversalNode): boolean;
    traverseResolution(name: string): boolean;
    traverseBranches(branches: TraversalEntry[][]): boolean;
}
export type TraversalConfig = {
    [k in keyof TypeConfig]-?: TypeConfig[k][];
};
export declare const traverse: (node: TraversalNode, state: TraversalState) => boolean;
export declare const checkEntries: (entries: TraversalEntry[], state: TraversalState) => boolean;
export declare const checkRequiredProp: (prop: TraversalProp, state: TraversalState<TraversableData>) => boolean;
export type ValidationTraversalKey = Exclude<TraversalKey, "morph">;
export type EntryChecker<k extends TraversalKey> = (constraint: TraversalValue<k>, state: TraversalState<RuleData<k>>) => boolean;
export type TraversableData = Record<string | number, unknown>;
export type ConstrainedRuleTraversalData = extend<{
    [k in TraversalKey]?: unknown;
}, {
    regex: string;
    divisor: number;
    bound: SizedData;
    prerequisiteProp: TraversableData;
    optionalProp: TraversableData;
    requiredProp: TraversableData;
    indexProp: TraversableData;
    distilledProps: TraversableData;
    strictProps: TraversableData;
}>;
export type RuleData<k extends TraversalKey> = k extends keyof ConstrainedRuleTraversalData ? ConstrainedRuleTraversalData[k] : unknown;
